// 
// abs2pcd.cpp : Convert FRGC 2D depth map (.abs) into a .pcd point cloud.
//
// Copyright (C) 2011  Clement Creusot
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <pcl/io/pcd_io.h>
#include <pcl/point_types.h>

#include <vector>
#include <fstream>

void ABSReader(const std::string& filename, pcl::PointCloud<pcl::PointXYZ>& cloud){
  
  std::ifstream in;
  std::string buff, name;
  
  in.open(filename.c_str());
  if (!in) {
    return ;
  }
  // Fill in the cloud data
  cloud.is_dense = false;

  in >> cloud.height;
  getline(in, buff); // read row ...\n
  in >> cloud.width;
  getline(in, buff); // read column ...\n
  getline(in, buff); // read "pixels (flag X Y Z):..\n"

  cloud.points.resize (cloud.width*cloud.height);
  std::vector< std::vector<int> > flags(cloud.height, std::vector<int>(cloud.width,0));
 
  // Read all the flags 
  for(int i=0; i<cloud.height; ++i){
    for(int j=0; j<cloud.width; ++j){
      in >> flags[i][j];
      if (flags[i][j] == 0){
	cloud.points[i*cloud.width+j].x = std::numeric_limits<double>::quiet_NaN ();
	cloud.points[i*cloud.width+j].y = std::numeric_limits<double>::quiet_NaN ();
	cloud.points[i*cloud.width+j].z = std::numeric_limits<double>::quiet_NaN ();
      }
    } 
  }

  // read Xs
  for(unsigned int i=0; i<cloud.height; ++i){
     for(unsigned int j=0; j<cloud.width; ++j){
       float temp;
       in >> temp;
       if (flags[i][j] == 1){
	 cloud.points[i*cloud.width+j].x = temp;
       }
    } 
  }

  // read Ys
  for(unsigned int i=0; i<cloud.height; ++i){
    for(unsigned int j=0; j<cloud.width; ++j){
      float temp;
      in >> temp;
       if (flags[i][j] == 1){
	 cloud.points[i*cloud.width+j].y = temp;
       }
    } 
  }

  // read Zs
  for(unsigned int i=0; i<cloud.height; ++i){
    for(unsigned int j=0; j<cloud.width; ++j){
      float temp;
      in >> temp;
      if (flags[i][j] == 1){
	 cloud.points[i*cloud.width+j].z = temp;
       }
    } 
  }

  //    
  // we have finished with the file
  in.close();
  return ;
}

void print_help(){
  fprintf(stderr, "Usage: abs2pcd filein.abs [fileout.pcd]\n");
  exit(EXIT_FAILURE);
}



int
  main (int argc, char** argv)
{
  std::string filein;
  std::string fileout;

  if (argc < 2){
    print_help();
  }
  else{
    filein = argv[1];
    fileout = argv[1];
    if (argc >= 3){
      fileout = argv[2]; 
    }  
    else{
      int end= fileout.rfind(".abs");
      fileout.replace(end,4, ".pcd");
    }
  }

  pcl::PointCloud<pcl::PointXYZ> cloud;
  ABSReader(filein, cloud);
  pcl::io::savePCDFileASCII (fileout, cloud);

  return (0);
}
